__version__ = "0.1.0"

from dvartk.parser import (
    SnvFileConfig,
    load_and_convert_snv_maf_columns,
    SnvComparison,
)

from dvartk.process import (
    count_snvs,
)

from dvartk.plotter import (
    plot_snv_spectra,
    plot_venn2,
)



import pandas as pd
import matplotlib
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib_venn import venn2, venn2_unweighted


def plot_snv_spectra(snv, title, save_path, tag="", despine=False, debug=False):
    fig, ax = plt.subplots(1)
    fig.set_figheight(3)
    fig.set_figwidth(20)

    if tag:
        title += tag
    fig.suptitle(title)

    pat = r"([ACGT])\[([ACGT])\>([ACGT])\]([ACGT])"
    df = pd.DataFrame(snv.copy(), columns=["count"])
    df["norm_tri_nt"] = df.index.str.replace(pat, r"\1\2\4", regex=True)
    df["norm_mut_type"] = df.index.str.replace(pat, r"\2>\3", regex=True)
    df["index"] = range(df.shape[0])
    if debug:
        print(df)

    for mut_type, mut_type_data in df.groupby(["norm_mut_type"]):
        if debug:
            print(mut_type, mut_type_data)
        ax.bar(data=mut_type_data, x="index", height="count", label=mut_type)

    font = matplotlib.font_manager.FontProperties()
    font.set_family("monospace")

    ax.set_xticks(df["index"])
    ax.set_xticklabels(df["norm_tri_nt"], rotation=90, fontproperties=font)
    ax.set_xlim((-1, 97))
    ax.legend(bbox_to_anchor=(1, 1), loc="upper left")

    sns.despine(ax=ax, trim=True)
    plt.tight_layout()

    if save_path:
        plt.savefig(save_path)


def plot_venn2(cmp, weighted=False, label1="A", label2="B"):
    """Draw a venn diagram from a Snv/SvComparison instance"""
    if weighted:
        vd = venn2([cmp.A, cmp.B], set_labels=(label1, label2))
    else:
        vd = venn2_unweighted([cmp.A, cmp.B], set_labels=(label1, label2))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dvartk']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'dvartk',
    'version': '0.1.2',
    'description': '',
    'long_description': '# dvartk (variant comparison toolkit)\n## Install\n```\npip install dvartk\n```\n\n## Philosophy\nBasically a package for comparing SNVs of maf files\n- in the future, SV, INDEL support, vcf support as well\n\n## Usage\n### Comparing SNVs\n```\nimport dvartk\n\n# set MAF paths\nmaf1_path = \'/path/to/maf1\'\nmaf2_path = \'/path/to/maf2\'\n\n# convert custom chrom, pos, ref, alt column names to "chrom", "pos", "ref", "alt"\nsnv_file_config = dvartk.parser.SnvFileConfig(\'Chromosome\', \'Start_Position\', \'Reference_Allele\', \'Tumor_Seq_Allele2\')\n\n# load and convert columns of MAF\nmaf1 = dvartk.load_and_convert_snv_maf_columns(maf1_path, snv_file_config)\nmaf2 = dvartk.load_and_convert_snv_maf_columns(maf2_path, snv_file_config)\n\n# get set counts (intersection, difference, ...) between maf1 and maf2\n# first make a SNV comparison instance\nsnv_cmp = SnvComparison(maf1, maf2)\nsnv_cmp.get_set_counts() # get A[maf1], B[maf2] counts\nsummary = snv_cmp.make_oneliner()\nprint(summary)\n```\n\n### Plot SNV trinucleotide spectra\n```\nimport dvartk\n\n# set a MAF path\nmaf_path = \'/path/to/maf\'\n\n# get counts per trinucleotide type\ncounts = dvartk.count_snvs(maf)\ndvartk.plot_snv_spectra(\n    counts,\n    \'plot title\',  # title of the plot e.g. sample ID\n    save_path=None,  # output path for your plot; if None, don\'t save plot\n    tag=\' (bar)\'  # tag to add to plot title; if None, don\'t add tag\n)\n```\n',
    'author': 'Seongmin Choi',
    'author_email': 'soymintc@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import pandas as pd


class SnvFileConfig:
    """Config with source and dest column names"""

    def __init__(self, chrom_src, pos_src, ref_src, alt_src):
        self.chrom_src = chrom_src
        self.pos_src = pos_src
        self.ref_src = ref_src
        self.alt_src = alt_src
        self.chrom_dst = "chrom"
        self.pos_dst = "pos"
        self.ref_dst = "ref"
        self.alt_dst = "alt"

        self.col_converter = {
            self.chrom_src: self.chrom_dst,
            self.pos_src: self.pos_dst,
            self.ref_src: self.ref_dst,
            self.alt_src: self.alt_dst,
        }


def load_and_convert_snv_maf_columns(maf_path, snv_file_config):
    """Load a maf, select SNPs only, then convert column names"""
    maf = pd.read_csv(
        maf_path, dtype={"Chromosome": str}, sep="\t", comment="#", low_memory=False
    )
    assert "Variant_Type" in maf.columns  # TODO: generalize
    maf = maf[maf["Variant_Type"] == "SNP"]  # TODO: generalize

    assert len(set(snv_file_config.col_converter.keys()) & set(maf.columns)) == len(
        snv_file_config.col_converter
    )  # all src names should be in maf columns
    maf = maf.rename(columns=snv_file_config.col_converter)
    return maf


class SnvComparison:
    """Class for comparing two 'maf' tables"""

    ixs = ["chrom", "pos", "ref", "alt"]

    def __init__(self, maf1, maf2, delimitor="\t", debug=False):
        self.maf1 = maf1.reset_index(drop=True)
        self.maf2 = maf2.reset_index(drop=True)
        self.delimitor = delimitor
        self.debug = debug

        if self.debug:
            print(f"self.maf1: {self.maf1}")
            print(f"self.maf2: {self.maf2}")

        snv_match = pd.merge(
            self.maf1[self.ixs], self.maf2[self.ixs], how="inner", on=self.ixs
        ).set_index(self.ixs)

        self.maf1 = self.maf1.set_index(self.ixs, drop=False)
        self.maf2 = self.maf2.set_index(self.ixs, drop=False)

        self.maf1_match = self.maf1[self.maf1.index.isin(snv_match.index)]
        self.maf1_nonmatch = self.maf1[~self.maf1.index.isin(snv_match.index)]
        self.maf2_match = self.maf2[self.maf2.index.isin(snv_match.index)]
        self.maf2_nonmatch = self.maf2[~self.maf2.index.isin(snv_match.index)]

    def make_set(self, data):
        df_ix = data.copy().set_index(self.ixs).index.tolist()
        df_ix_set = set(df_ix)
        return df_ix_set

    def get_set_counts(self):
        self.A_and_B = self.make_set(
            self.maf1_match
        )  # big assumption that results will be same/similar with self.maf2_match
        self.A_and_B_from_maf2 = self.make_set(
            self.maf2_match
        )  # for twisted people like Seongmin who can't trust himself
        self.A_not_B = self.make_set(self.maf1_nonmatch)
        self.B_not_A = self.make_set(self.maf2_nonmatch)
        self.A = self.A_and_B.union(self.A_not_B)
        self.B = self.A_and_B.union(self.B_not_A)

    def make_oneliner(self, name=None, get_str=False):
        for attr in ["A", "B"]:
            assert hasattr(self, attr)
        field = [
            len(self.A),
            len(self.B),
            len(self.A - self.B),
            len(self.B - self.A),
            len(self.A & self.B),
            len(self.A | self.B),
        ]
        if name:
            field = [name] + field
        field = [str(_) for _ in field]
        if get_str:
            return self.delimitor.join(field)
        return field



import pandas as pd
import numpy as np
from pyfaidx import Fasta

genome = Fasta("/juno/work/shah/users/chois7/mmctm/reference/GRCh37-lite.fa")


def construct_empty_count_series():
    snvs = ["C>A", "C>G", "C>T", "T>A", "T>C", "T>G"]
    nts = ["A", "C", "G", "T"]
    terms = ["{}[{}]{}".format(l, s, r) for s in snvs for l in nts for r in nts]
    return pd.Series(np.zeros(len(terms), dtype=int), index=terms)


def normalize_snv(context, alt):
    ref = context.seq[1]

    if ref in ["A", "G"]:
        context = (-context).seq

        complement = {"A": "T", "C": "G", "G": "C", "T": "A"}
        alt = complement[str(alt)]
    else:
        context = context.seq
        alt = str(alt)
    return context, alt


def construct_snv_label(context, alt):
    if len(context) != 3:
        warnings.warn("Warning: bad context length: {}".format(str(context)))
        return None
    return "{}[{}>{}]{}".format(context[0], context[1], alt, context[2])


def count_snvs(snvs, genome=genome):
    """Convert maf form to count table per variant type. Requires 'genome'"""
    df = snvs.copy()  # snvs <- essentially "maf" variable

    var_converter = {
        "G>T": "C>A",
        "G>C": "C>G",
        "G>A": "C>T",
        "A>T": "T>A",
        "A>G": "T>C",
        "A>C": "T>G",
    }
    df["var_type"] = (df["ref"] + ">" + df["alt"]).replace(var_converter)

    counts = construct_empty_count_series()
    contexts = []

    for idx, row in snvs.iterrows():
        # two flanking bases
        start = row["pos"] - 2
        end = row["pos"] + 1

        context = genome[row["chrom"]][start:end]
        if "N" in context.seq:
            warnings.warn(
                "Warning: N found in context sequence at {}:{}-{}".format(
                    row["chrom"], start + 1, end
                )
            )
            continue

        context, alt = normalize_snv(context, row["alt"])
        counts[construct_snv_label(context, alt)] += 1
        contexts.append(context)

    assert snvs.shape[0] == counts.sum()

    return counts
